//
// File: IPv4Socket.h
// Created by: Kristian Müller <Kristian-M@Kristian-M.de>
// Created on: Wed Oct  1 15:18:38 2003
//

#include "../include/IPv4Socket.h"
// #include <iostream.h> //debugging

IPv4Socket::IPv4Socket(char *sInHostname, int iInIPv4Port){
	sIPv4Address = NULL;
	sIPv4Hostname = NULL;
	char *sTempIPv4Hostname = NULL;
	
	iIPv4Port = iInIPv4Port;
	bConnected = false;
	if (sInHostname == NULL) return;  // very bad situation!!
	sIPv4Hostname = new char[strlen(sInHostname)+1];
	strncpy(sIPv4Hostname, sInHostname, strlen(sInHostname));
	sIPv4Hostname[strlen(sInHostname)] = '\0';
	sIPv4Address = new char[16];
	sTempIPv4Hostname = resolveHost(sIPv4Hostname);
	if (sTempIPv4Hostname == NULL){
		sIPv4Address = NULL;
	} else {
	  strncpy (sIPv4Address, sTempIPv4Hostname, 15);
		sIPv4Address[15] = '\0';
	}
	aSocketAddress.sin_family	= AF_INET;
	aSocketAddress.sin_port	= htons(iInIPv4Port);
	iIPv4SocketDescriptor = socket(AF_INET,SOCK_DGRAM,IPPROTO_UDP);
};

IPv4Socket::~IPv4Socket(){
	if (sIPv4Address != NULL) delete[] sIPv4Address;
	if (sIPv4Hostname != NULL) delete[] sIPv4Hostname;
}

int IPv4Socket::setPort(int iInIPv4Port){
	if (bConnected) return -1;
  iIPv4Port = iInIPv4Port;
	aSocketAddress.sin_port	= htons(iInIPv4Port);
}

char *IPv4Socket::resolveHost(char *sInHostName){
	ErrorHandler aErrorHandler;
  struct hostent *aResolvedEntry = NULL;
	
	if (inet_aton(sInHostName, &aSocketAddress.sin_addr) != 1) {
		if ((aResolvedEntry = gethostbyname(sInHostName)) == NULL) {
			char *sErrorString = new char[1024];
			sprintf(sErrorString, "Could not resolve the hostname of %s \nMaybe you should use its IP directly.", sInHostName);
			aErrorHandler.showError(sErrorString);
			return NULL;
		}
		memcpy(&aSocketAddress.sin_addr,
					 aResolvedEntry->h_addr,
					 sizeof(aSocketAddress.sin_addr));			
		return (char *)&aSocketAddress.sin_addr;
	} else {
		return NULL;
	}
}


int IPv4Socket::connectSocket(){
	ErrorHandler aErrorHandler;
	
	if (connect(iIPv4SocketDescriptor,(struct sockaddr *)&aSocketAddress,sizeof(aSocketAddress)) != 0){
		char *sErrorString = new char[1024];
		sprintf(sErrorString, "Error while connecting to the socket at %s:%i.", sIPv4Hostname, iIPv4Port);
		aErrorHandler.showError(sErrorString);
		bConnected = false;
		return -1;
	}
	bConnected = true;
	return 1;
}


int IPv4Socket::writeToSocket(void *sPacket, int iPacketLenght) {
	ErrorHandler aErrorHandler;
	
	if (!bConnected) 
		connectSocket();
	if (!bConnected)
		return -1;
	if (write(iIPv4SocketDescriptor, sPacket, iPacketLenght) != iPacketLenght){
		char *sErrorString;
		sprintf(sErrorString, "Error while writing to the socket at %s:%i.", 
						sIPv4Address, iIPv4Port);
		aErrorHandler.showError(sErrorString);
		return -1;
	}
	return 1;
}


int IPv4Socket::readFromSocket(void *sPacket, int iPacketLenght) {
	fd_set aSocketDescriptorSet;
	ErrorHandler aErrorHandler;
	struct timeval aTimeoutValue = {3,0};
	int iReadReturnValue;

	if (!bConnected) 
		connectSocket();
	FD_ZERO(&aSocketDescriptorSet);
	FD_SET(iIPv4SocketDescriptor,&aSocketDescriptorSet);
	switch(select(iIPv4SocketDescriptor + 1,&aSocketDescriptorSet,
		     NULL,NULL,&aTimeoutValue)){
		case 1:
		  if ((iReadReturnValue = read(iIPv4SocketDescriptor,sPacket,iPacketLenght)))
				break;
	  	aErrorHandler.showError("Failed to read packet.");
			return -1;
		case 0:
			errno	= ETIMEDOUT;
		default:
	  	aErrorHandler.showError("Failed to read packet.");
			return -1;
	}
/*	
	if (iReadReturnValue != iPacketLenght)
		aErrorHandler.showError("requested %d bytes, received packet of %d bytes",len,r);
*/
	return 1;
}
